@@ -186,7 +186,7 @@ Win32System(const char *cmdline)
 
 #define NX_XKB_BASE_DIRECTORY            "/usr/lib/X11/xkb"
 #define NX_XKB_ALTERNATE_BASE_DIRECTORY  "/usr/share/X11/xkb"
-#define NX_KEYMAP_DIR_FILE               "keymap.dir"
+#define NX_XKB_RULES_BASE_FILE           "rules/base"
 #define NX_ALT_XKBCOMP_PATH              "/usr/bin"
 
 static char _NXXkbBasePath[PATH_MAX];
@@ -195,43 +195,43 @@ static char _NXXkbCompPath[PATH_MAX];
 static int NXVerifyXkbBaseDirectory(const char *dirPath)
 {
   int size;
-  char *keymapDirFilePath;
-  struct stat keymapDirFileStat;
+  char *rulesBaseFilePath;
+  struct stat rulesBaseFileStat;
 
   /*
-   * If keymap.dir file
-   * is not present into
-   * Xkb Base Directory,
+   * If rules/base file
+   * is not present inside
+   * the Xkb Base Directory,
    * we suppose that the
    * path is not valid.
    */
 
   size = strlen(dirPath) + strlen("/") +
-             strlen(NX_KEYMAP_DIR_FILE) + 1;
+             strlen(NX_XKB_RULES_BASE_FILE) + 1;
 
-  if ((keymapDirFilePath = malloc((size + 1) * sizeof(char))) == NULL)
+  if ((rulesBaseFilePath = malloc((size + 1) * sizeof(char))) == NULL)
   {
     FatalError("NXVerifyXkbBaseDirectory: malloc failed.\n");
   }
 
-  strcpy(keymapDirFilePath, dirPath);
-  strcat(keymapDirFilePath, "/");
-  strcat(keymapDirFilePath, NX_KEYMAP_DIR_FILE);
+  strcpy(rulesBaseFilePath, dirPath);
+  strcat(rulesBaseFilePath, "/");
+  strcat(rulesBaseFilePath, NX_XKB_RULES_BASE_FILE);
 
   #ifdef TEST
   fprintf(stderr, "NXVerifyXkbBaseDirectory: Looking for [%s] file.\n",
-              keymapDirFilePath);
+              rulesBaseFilePath);
   #endif
 
-  if (stat(keymapDirFilePath, &keymapDirFileStat) != 0)
+  if (stat(rulesBaseFilePath, &rulesBaseFileStat) != 0)
   {
 
     #ifdef TEST
     fprintf(stderr, "NXVerifyXkbBaseDirectory: Can't find the keymap.dir file [%s].\n",
-                keymapDirFilePath);
+                rulesBaseFilePath);
     #endif
 
-    free(keymapDirFilePath);
+    free(rulesBaseFilePath);
 
     return 0;
   }
@@ -241,7 +241,7 @@ static int NXVerifyXkbBaseDirectory(const char *dirPath)
               dirPath);
   #endif
 
-  free(keymapDirFilePath);
+  free(rulesBaseFilePath);
 
   return 1;
 }
